task "script" "go_fmt" {
    inline = [
        "go fmt ${join(" ", toset([for s in vcs.staged_files : "code.icb4dc0.de/buildr/buildr/${dirname(s)}" if ext(s) == ".go"]))}"
    ]
}

task "script" "go_generate" {
    inline = [
        "go generate -x ./..."
    ]

    environment = {
        GOWORK = "off"
    }

    depends_on = [
        tools.mockery.id
    ]
}

task "script" "go_test" {
    inline = [
        "apt update && apt install -y fuse",
        "go run -mod=mod gotest.tools/gotestsum --junitfile results.xml --format pkgname-and-test-fails -- -shuffle=on ./..."
    ]

    environment = {
        GOWORK = "off"
    }

    depends_on = [
        tasks.go_generate.id
    ]

    input_mapping = {
        "${repo.root}" = "."
    }

    container {
        image      = "docker.io/golang:1.21"
        privileged = true

        volume_mount "/go/pkg/mod" {
            name = "go_cache"
        }

        bind_mount "/dev/fuse" {
            source = "/dev/fuse"
        }
    }
}

task "script" "golangci_lint" {
    inline = [
        "golangci-lint run -v"
    ]

    container {
        image = "golangci/golangci-lint"

        volume_mount "/root/.cache/golangci-lint" {
            name = "golangci_lint_cache"
        }

        volume_mount "/go/pkg/mod" {
            name = "go_cache"
        }
    }

    depends_on = [
        tasks.go_fmt.id,
        tasks.go_generate.id,
    ]

    input_mapping = {
        "${repo.root}" = "."
    }
}