package cmd

import (
	"fmt"
	"io"
	"os"
	"strings"
)

var _ EnvCommander = (*EnvApp)(nil)

func NewEnvApp(accessor BuildrConfigAccessor) *EnvApp {
	return &EnvApp{
		executionSpecAccessor: accessor,
	}
}

type EnvApp struct {
	executionSpecAccessor BuildrConfigAccessor
}

func (e EnvApp) PrintPath(writer io.Writer) error {
	execSpec, err := e.executionSpecAccessor.BuildrConfig()
	if err != nil {
		return err
	}
	currentPath, ok := os.LookupEnv("PATH")
	if !ok {
		_, err := fmt.Fprintf(writer, "PATH=%s", execSpec.BinariesDirectory)
		return err
	}

	joinedPath := strings.Join([]string{execSpec.BinariesDirectory, currentPath}, string(os.PathListSeparator))
	_, err = fmt.Fprintf(writer, "PATH=%s", joinedPath)
	return err
}
