package cmd

import (
	"context"
	"io"

	"code.icb4dc0.de/buildr/buildr/internal/plugins"

	"github.com/olekukonko/tablewriter"
)

var _ PluginCommander = (*PluginApp)(nil)

func NewPluginApp(buildrCfgAccessor BuildrConfigAccessor, pluginMgr *plugins.Manager, svcAcc PluginsRepoAccessor) *PluginApp {
	return &PluginApp{
		buildrCfgAccessor: buildrCfgAccessor,
		pluginMgr:         pluginMgr,
		svcAcc:            svcAcc,
	}
}

type PluginApp struct {
	buildrCfgAccessor BuildrConfigAccessor
	pluginMgr         *plugins.Manager
	svcAcc            PluginsRepoAccessor
}

func (p PluginApp) ListPlugins(ctx context.Context, writer io.Writer) error {
	repo, err := p.svcAcc.PluginsRepo()
	if err != nil {
		return err
	}
	knownPlugins, err := repo.List(ctx)
	if err != nil {
		return err
	}
	table := tablewriter.NewWriter(writer)
	table.SetHeader([]string{"Name"})
	for _, plugin := range knownPlugins {
		table.Append([]string{plugin.Name})
	}

	table.Render()
	return nil
}

func (p PluginApp) UpdatePlugins(ctx context.Context) error {
	cfg, err := p.buildrCfgAccessor.BuildrConfig()
	if err != nil {
		return err
	}
	return p.pluginMgr.UpdatePlugins(ctx, cfg.Plugins...)
}
