package cmd

import (
	"context"
	"log/slog"

	"code.icb4dc0.de/buildr/buildr/internal/rpc"
)

var _ ServerCommander = (*ServerApp)(nil)

func NewServerApp(accessor TypeRegistryAccessor) *ServerApp {
	return &ServerApp{
		accessor: accessor,
	}
}

type ServerApp struct {
	accessor TypeRegistryAccessor
}

func (s *ServerApp) ServeAPI(ctx context.Context, cfg *rpc.GrpcConfig) error {
	logger := slog.Default()

	logger.Info("Starting gRPC server", slog.Group("grpc", slog.String("addr", cfg.Host.Address)))

	grpcServer := rpc.NewServer(logger, s.accessor)

	go func() {
		if err := grpcServer.Start(ctx, cfg.Host.Address); err != nil {
			slog.Error("Error occurred while serving gRPC API", slog.String("err", err.Error()))
		}
	}()

	<-ctx.Done()

	//nolint:contextcheck // context is already done when reaching this point
	grpcServer.Close()

	return nil
}
