package cmd

import (
	"strings"

	"github.com/spf13/cobra"

	"code.icb4dc0.de/buildr/buildr/internal/maps"
	"code.icb4dc0.de/buildr/buildr/modules"
)

func ModulesArgsProviderFor(
	registryAcc TypeRegistryAccessor,
	cat modules.Category,
) KnownModulesArgProvider {
	return KnownModulesArgProviderFunc(func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
		if len(args) != 0 {
			return nil, cobra.ShellCompDirectiveNoFileComp
		}

		knownModules := registryAcc.TypeRegistry().Inventory()[cat]
		filtered := make([]string, 0, len(knownModules))
		toComplete = strings.ToLower(toComplete)

		for i := range knownModules {
			if strings.HasPrefix(strings.ToLower(knownModules[i]), toComplete) {
				filtered = append(filtered, knownModules[i])
			}
		}

		return filtered, cobra.ShellCompDirectiveNoFileComp
	})
}

func TasksArgsProviderFor(repoAcc ModuleRepositoryAccessor, cat modules.Category) KnownTasksArgProvider {
	return KnownTasksArgProviderFunc(func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
		if len(args) != 0 {
			return nil, cobra.ShellCompDirectiveNoFileComp
		}

		repo, err := repoAcc.Repository()
		if err != nil {
			return nil, cobra.ShellCompDirectiveError
		}

		tasks := maps.Keys(repo.ModulesByCategory(cat))

		filtered := make([]string, 0, len(tasks))

		for i := range tasks {
			if strings.HasPrefix(tasks[i], toComplete) {
				filtered = append(filtered, tasks[i])
			}
		}

		return filtered, cobra.ShellCompDirectiveNoFileComp
	})
}
