package cmd

import (
	"github.com/spf13/cobra"

	"code.icb4dc0.de/buildr/buildr/modules"
)

func NewCmd(subCommands ...*cobra.Command) *cobra.Command {
	newCmd := &cobra.Command{
		Use:   "new",
		Short: "Bootstrap modules",
	}

	newCmd.AddCommand(subCommands...)

	return newCmd
}

func BootstrapModuleCmd(
	category modules.Category,
	cmder BootstrapModuleCommander,
	argsProvider KnownModulesArgProvider,
	opts ...ModuleCommandOption,
) *cobra.Command {
	const argsWithModuleName = 2
	cmd := &cobra.Command{
		Use:               modules.CategoryName(category),
		SilenceUsage:      true,
		SilenceErrors:     true,
		ValidArgsFunction: argsProvider.ValidModulesArgs,
		Args:              cobra.RangeArgs(1, argsWithModuleName),
		RunE: func(cmd *cobra.Command, args []string) error {
			var (
				typeName   = args[0]
				moduleName string
			)

			if len(args) == argsWithModuleName {
				moduleName = args[1]
			}

			return cmder.BootstrapModule(cmd.Context(), category, typeName, moduleName)
		},
	}

	for i := range opts {
		opts[i](cmd)
	}

	return cmd
}
