package cmd

import (
	"os"

	"github.com/spf13/cobra"
)

func PluginListCommand(cmder PluginCommander) *cobra.Command {
	cmd := &cobra.Command{
		Use:     "list",
		Short:   "List plugins",
		Aliases: []string{"ls", "dir"},
		RunE: func(cmd *cobra.Command, args []string) error {
			return cmder.ListPlugins(cmd.Context(), os.Stdout)
		},
	}

	return cmd
}

func PluginUpdateCommand(cmder PluginCommander) *cobra.Command {
	cmd := &cobra.Command{
		Use:   "update",
		Short: "Update plugins",
		RunE: func(cmd *cobra.Command, args []string) error {
			return cmder.UpdatePlugins(cmd.Context())
		},
	}

	return cmd
}

func PluginsCommand(cmder PluginCommander) *cobra.Command {
	cmd := &cobra.Command{
		Use:           "plugins",
		Short:         "Manage plugins",
		SilenceUsage:  true,
		SilenceErrors: true,
	}

	cmd.AddCommand(PluginListCommand(cmder), PluginUpdateCommand(cmder))

	return cmd
}
