package cmd

import (
	"strings"

	"github.com/spf13/cobra"

	"code.icb4dc0.de/buildr/buildr/modules"
)

func RunCommand(
	mra ModuleRepositoryAccessor,
	moduleExecCmder ModuleCommander,
	toolExecCmder RunToolCommander,
) *cobra.Command {
	return &cobra.Command{
		Use:               "run",
		Aliases:           []string{"exec"},
		Short:             "Run a tool - tool will be installed on demand",
		SilenceUsage:      true,
		SilenceErrors:     true,
		Args:              cobra.MinimumNArgs(1),
		ValidArgsFunction: completeTools(mra),
		RunE: func(cmd *cobra.Command, args []string) error {
			if err := moduleExecCmder.RunModule(cmd.Context(), modules.CategoryTool, args[0]); err != nil {
				return err
			}

			return toolExecCmder.RunTool(cmd.Context(), args[0], args[1:])
		},
	}
}

//nolint:lll // return type is what it is
func completeTools(mra ModuleRepositoryAccessor) func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	return func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
		if len(args) > 0 {
			return nil, cobra.ShellCompDirectiveDefault
		}

		toComplete = strings.ToLower(toComplete)

		repo, err := mra.Repository()
		if err != nil {
			return nil, cobra.ShellCompDirectiveError
		}

		tools := repo.ModulesByCategory(modules.CategoryTool)

		completions := make([]string, 0, len(tools))
		for _, t := range tools {
			toolName := t.Name()
			if bn, ok := t.Unwrap().(modules.BinaryNamer); ok {
				if name, err := bn.BinaryName(cmd.Context()); err != nil {
					continue
				} else {
					toolName = name
				}
			}

			if strings.HasPrefix(toolName, toComplete) {
				completions = append(completions, toolName)
			}
		}

		return completions, cobra.ShellCompDirectiveNoFileComp
	}
}
