package container

import (
	"fmt"

	"code.icb4dc0.de/buildr/buildr/internal/lazy"
	"code.icb4dc0.de/buildr/buildr/modules/state"

	"code.icb4dc0.de/buildr/buildr/internal/containers"
	"code.icb4dc0.de/buildr/buildr/internal/execution"
	"code.icb4dc0.de/buildr/buildr/modules"
)

var _ execution.TaskProvider = (*taskProvider)(nil)

func Provider(
	orchestratorLazy *lazy.Lazy[*containers.Orchestrator],
	repo *modules.Repository,
	stateStore state.Store,
) execution.TaskProvider {
	return taskProvider{
		orchestrator: orchestratorLazy,
		repo:         repo,
		stateStore:   stateStore,
	}
}

type taskProvider struct {
	orchestrator *lazy.Lazy[*containers.Orchestrator]
	repo         *modules.Repository
	stateStore   state.Store
}

func (taskProvider) CanProvide(m modules.ModuleWithMeta) bool {
	return m.ContainerSpec() != nil
}

func (p taskProvider) Create(m modules.ModuleWithMeta) (execution.Task, error) {
	orchestrator, err := p.orchestrator.Get()
	if err != nil {
		return nil, err
	}

	if m.ContainerSpec() == nil {
		return nil, fmt.Errorf("module %v doesn't have a container spec - should be handled differently", m.Name())
	} else {
		return &containerTask{
			moduleWithMeta: m,
			orchestrator:   orchestrator,
			repo:           p.repo,
			stateStore:     p.stateStore,
		}, nil
	}
}
