package ignore_test

import (
	"os"
	"testing"

	"code.icb4dc0.de/buildr/buildr/internal/ignore"
)

func TestIgnorer_Match(t *testing.T) {
	t.Parallel()
	cwd, err := os.Getwd()
	if err != nil {
		t.Fatal(err)
	}
	type fields struct {
		Parent   *ignore.Ignorer
		Patterns []string
	}
	type args struct {
		path string
	}
	tests := []struct {
		name   string
		args   args
		fields fields
		want   bool
	}{
		{
			name: "Match .git dir",
			fields: fields{
				Patterns: []string{
					".git/",
				},
			},
			args: args{
				path: "/work/asdf/buildr/.git/objects",
			},
			want: true,
		},
		{
			name: "Match log file on dir base",
			fields: fields{
				Parent: ignore.MustNewIgnorer(cwd, ".git"),
				Patterns: []string{
					"logs/",
					"out/",
					"bin",
				},
			},
			args: args{
				path: "/work/asdf/buildr/.buildr/logs/task_buf_generate.stderr.log",
			},
			want: true,
		},
		{
			name: "Match log file on extension base",
			fields: fields{
				Parent: ignore.MustNewIgnorer(cwd, ".git"),
				Patterns: []string{
					"*.log",
				},
			},
			args: args{
				path: "/work/asdf/buildr/.buildr/logs/task_buf_generate.stderr.log",
			},
			want: true,
		},
		{
			name: "Match tool based on dir",
			fields: fields{
				Parent: ignore.MustNewIgnorer(cwd, ".git"),
				Patterns: []string{
					"logs/",
					"out/",
					"bin/",
				},
			},
			args: args{
				path: "/work/asdf/buildr/.buildr/bin/mockery",
			},
			want: true,
		},
	}
	for _, tt := range tests {
		tt := tt
		t.Run(tt.name, func(t *testing.T) {
			t.Parallel()
			ig, err := ignore.NewIgnorer(cwd, tt.fields.Patterns...)
			if err != nil {
				t.Fatalf("Failed to create ignorer: %v", err)
			}

			ig.SetParent(tt.fields.Parent)

			if got := ig.Ignore(tt.args.path); got != tt.want {
				t.Errorf("Match() got = %v, want %v", got, tt.want)
			}
		})
	}
}
