package ioutils

import (
	"io"
	"sync"
)

const defaultBufferSize = 256 * 1024

var bufferPool = sync.Pool{
	New: func() any {
		return make([]byte, defaultBufferSize)
	},
}

func CopyWithPooledBuffer(dst io.Writer, src io.Reader) (n int64, err error) {
	buffer := bufferPool.Get().([]byte)
	//nolint:staticcheck // should be fine for slices
	defer bufferPool.Put(buffer)

	n, err = io.CopyBuffer(dst, src, buffer)
	if err != nil {
		return
	}

	return
}
