package lazy_test

import (
	"testing"
	"time"

	"code.icb4dc0.de/buildr/buildr/internal/lazy"
)

const sampleValue = "Hello, world"

func TestLazy_Get_Once(t *testing.T) {
	t.Parallel()
	l := lazy.NewLazy(func() (string, error) {
		return sampleValue, nil
	})

	v, err := l.Get()
	if err != nil {
		t.Fatal(err)
	}
	if v != sampleValue {
		t.Fatalf("expected %q, got %q", sampleValue, v)
	}
}

func TestLazy_Get_MultipleTimes(t *testing.T) {
	t.Parallel()
	l := lazy.NewLazy(func() (string, error) {
		return sampleValue, nil
	})

	for i := 0; i < 10; i++ {
		v, err := l.Get()
		if err != nil {
			t.Fatal(err)
		}
		if v != sampleValue {
			t.Fatalf("expected %q, got %q", sampleValue, v)
		}
	}
}

func TestLazy_Get_AsyncProcessing(t *testing.T) {
	t.Parallel()
	l := lazy.NewLazy(func() (string, error) {
		time.Sleep(100 * time.Millisecond)

		return "Async hello, world!", nil
	})

	do := func(f func() (string, error)) (time.Duration, error) {
		start := time.Now()
		_, err := f()
		return time.Since(start), err
	}

	duration, err := do(l.Get)
	if err != nil {
		t.Fatal(err)
	}

	if duration > 150*time.Millisecond {
		t.Errorf("expected duration to be less than 150ms, got %v", duration)
	}

	for i := 0; i < 10; i++ {
		duration, err := do(l.Get)
		if err != nil {
			t.Fatal(err)
		}

		if duration > 10*time.Millisecond {
			t.Errorf("expected duration to be less than 10ms, got %v", duration)
		}
	}
}
