package storage

import (
	"syscall"

	unionfs "github.com/hanwen/go-fuse/v2/fs"
)

func (u *UnionFSNode) idFromStat(st *syscall.Stat_t) unionfs.StableAttr {
	// We compose an inode number by the underlying inode, and
	// mixing in the device number. In traditional filesystems,
	// the inode numbers are small. The device numbers are also
	// small (typically 16 bit). Finally, we mask out the root
	// device number of the root, so a loopback FS that does not
	// encompass multiple mounts will reflect the inode numbers of
	// the underlying filesystem
	swapped := (st.Dev << 32) | (st.Dev >> 32)
	return unionfs.StableAttr{
		Mode: st.Mode,
		Gen:  1,
		// This should work well for traditional backing FSes,
		// not so much for other go-fuse FS-es
		Ino: swapped ^ st.Ino,
	}
}
