package vault

import (
	"errors"

	"github.com/zclconf/go-cty/cty"
	"github.com/zclconf/go-cty/cty/function"
)

func FromVault(getter VaultGetter) function.Function {
	return function.New(&function.Spec{
		Description: "Get value from vault",
		Params: []function.Parameter{
			{
				Name:        "vaultKey",
				Description: "Key for lookup in vault",
				Type:        cty.String,
			},
		},
		Type: function.StaticReturnType(cty.String),
		Impl: func(args []cty.Value, retType cty.Type) (cty.Value, error) {
			if getter == nil {
				return cty.StringVal(""), errors.New("vault is not initialized - cannot read value")
			}
			val, err := getter.GetValue(args[0].AsString())
			if err != nil {
				return cty.Value{}, err
			}

			return cty.StringVal(string(val)), nil
		},
	})
}
