package archive_test

import (
	"context"
	"path/filepath"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"

	"code.icb4dc0.de/buildr/buildr/modules"
	"code.icb4dc0.de/buildr/buildr/modules/packaging/archive"
	"code.icb4dc0.de/buildr/buildr/modules/state"
)

func TestGzipFileCompression_SampleWithoutOutFile(t *testing.T) {
	t.Parallel()
	f := createSampleFile(t, 500*1024)

	gzip := archive.GzipFileCompression{Source: f}

	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
	t.Cleanup(cancel)

	outDir := t.TempDir()

	execCtx := modules.TestExecutionContext{
		Context:         ctx,
		TB:              t,
		OutputDirectory: outDir,
		StateStore:      new(state.InMemoryStore),
	}

	if !assert.NoError(t, gzip.Execute(execCtx), "failed to execute gzip module") {
		t.FailNow()
	}

	files, err := filepath.Glob(filepath.Join(outDir, "*.gz"))
	if !assert.NoError(t, err, "failed to glob files") {
		t.FailNow()
	}

	assert.Len(t, files, 1)
}

func TestGzipFileCompression_SampleWithOutFile(t *testing.T) {
	t.Parallel()
	f := createSampleFile(t, 500*1024)

	outFile := filepath.Join(t.TempDir(), "out.gz")

	gzip := archive.GzipFileCompression{Source: f, OutFile: outFile}

	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
	t.Cleanup(cancel)

	execCtx := modules.TestExecutionContext{
		Context:    ctx,
		TB:         t,
		StateStore: new(state.InMemoryStore),
	}

	if !assert.NoError(t, gzip.Execute(execCtx), "failed to execute gzip module") {
		t.FailNow()
	}

	assert.FileExists(t, outFile)
}

func TestGzipFileCompression_SampleWithRelativeOutFile(t *testing.T) {
	t.Parallel()
	f := createSampleFile(t, 500*1024)

	outDir := t.TempDir()

	gzip := archive.GzipFileCompression{Source: f, OutFile: "out.gz"}

	ctx, cancel := context.WithTimeout(context.Background(), 1*time.Second)
	t.Cleanup(cancel)

	execCtx := modules.TestExecutionContext{
		Context:         ctx,
		TB:              t,
		StateStore:      new(state.InMemoryStore),
		OutputDirectory: outDir,
	}

	if !assert.NoError(t, gzip.Execute(execCtx), "failed to execute gzip module") {
		t.FailNow()
	}

	assert.FileExists(t, filepath.Join(outDir, "out.gz"))
}
