package ociimg

import "github.com/google/go-containerregistry/pkg/authn"

var (
	_ authn.Keychain      = (*BuildrKeyChain)(nil)
	_ authn.Authenticator = (*AuthConfig)(nil)
)

type RegistryAuth struct {
	Registry   string `hcl:",label"`
	AuthConfig `hcl:",remain"`
}

func KeyChain(creds []RegistryAuth) BuildrKeyChain {
	chain := make(BuildrKeyChain)
	for i := range creds {
		c := creds[i]
		chain[c.Registry] = c.AuthConfig
	}

	return chain
}

type BuildrKeyChain map[string]AuthConfig

func (b BuildrKeyChain) Resolve(resource authn.Resource) (authn.Authenticator, error) {
	if cfg, ok := b[resource.RegistryStr()]; !ok {
		return authn.Anonymous, nil
	} else {
		return cfg, nil
	}
}

type AuthConfig struct {
	Username string `hcl:"username,optional"`
	Password string `hcl:"password,optional"`
	Auth     string `hcl:"auth,optional"`

	// IdentityToken is used to authenticate the user and get
	// an access token for the registry.
	IdentityToken string `hcl:"identitytoken,optional"`

	// RegistryToken is a bearer token to be sent to a registry
	RegistryToken string `hcl:"registrytoken,optional"`
}

func (a AuthConfig) Authorization() (*authn.AuthConfig, error) {
	return &authn.AuthConfig{
		Username:      a.Username,
		Password:      a.Password,
		Auth:          a.Auth,
		IdentityToken: a.IdentityToken,
		RegistryToken: a.RegistryToken,
	}, nil
}
