package modules

type Repository struct {
	modulesByID  map[string]ModuleWithMeta
	moduleByName map[moduleSpec]ModuleWithMeta
}

func (s *Repository) RegisterModules(modules ...ModuleWithMeta) {
	if s.moduleByName == nil {
		s.moduleByName = make(map[moduleSpec]ModuleWithMeta, len(modules))
	}

	if s.modulesByID == nil {
		s.modulesByID = make(map[string]ModuleWithMeta, len(modules))
	}

	for i := range modules {
		module := modules[i]
		s.moduleByName[specOf(module.Category(), module.Name())] = module
		s.modulesByID[module.ID()] = module
	}
}

func (s *Repository) Tools() map[string]ModuleWithMeta {
	return s.ModulesByCategory(CategoryTool)
}

func (s *Repository) Tasks() map[string]ModuleWithMeta {
	return s.ModulesByCategory(CategoryTask)
}

func (s *Repository) Builds() map[string]ModuleWithMeta {
	return s.ModulesByCategory(CategoryBuild)
}

func (s *Repository) ModuleByID(id string) ModuleWithMeta {
	module, ok := s.modulesByID[id]
	if !ok {
		return nil
	}

	return module
}

func (s *Repository) Module(moduleType Category, moduleName string) ModuleWithMeta {
	module, ok := s.moduleByName[specOf(moduleType, moduleName)]
	if !ok {
		return nil
	}

	return module
}

func (s *Repository) ModulesByCategory(moduleType Category) map[string]ModuleWithMeta {
	out := make(map[string]ModuleWithMeta, len(s.moduleByName))
	for spec, module := range s.moduleByName {
		if spec.TypeName == moduleType {
			out[spec.ModuleName] = module
		}
	}

	return out
}

func specOf(typeName Category, moduleName string) moduleSpec {
	return moduleSpec{
		TypeName:   typeName,
		ModuleName: moduleName,
	}
}

type moduleSpec struct {
	ModuleName string
	TypeName   Category
}
