package task_test

import (
	"context"
	"os"
	"testing"

	"code.icb4dc0.de/buildr/buildr/modules"
	"code.icb4dc0.de/buildr/buildr/modules/state"
	"code.icb4dc0.de/buildr/buildr/modules/task"
)

func TestExamples(t *testing.T) {
	t.Parallel()
	cwd, err := os.Getwd()
	if err != nil {
		t.Fatal(err)
	}
	help, err := new(task.ScriptTask).Help(context.Background())
	if err != nil {
		t.Fatal(err)
	}

	for _, e := range help.Examples {
		e := e
		t.Run(e.Name, func(t *testing.T) {
			t.Parallel()
			var memStateStore state.InMemoryStore
			ctx, cancel := context.WithCancel(context.Background())
			defer cancel()
			execCtx := modules.TestExecutionContext{
				Context:    ctx,
				TB:         t,
				StateStore: &memStateStore,
			}

			if e.Spec.Unwrap().(task.ScriptTask).WorkingDir != "" {
				execCtx.TestWorkingDir = cwd
			}

			if err := e.Spec.Execute(execCtx); err != nil {
				t.Errorf("failed to execute example: %s: %v", e.Name, err)
			}
		})
	}
}
